#ifndef PROTOCOL_H
#define PROTOCOL_H


#include "PData.h"

/*!
 * \class Protocol Protocol.h
 *
 * \brief Base class for a communication protocol imlpementation
 *
 */
class Protocol
{
public:
    /*!
     * It performs the protocol byte by byte.
     *
     * \param byte Byte to be parsed according to the current protocol.
     *
     * \return It returns 1 when the processing of the byte leads to a
     * completion of some statement of the protocol. It return 0 otherwise.
     */
    virtual bool parseByte (unsigned char byte) = 0;

    /*!
     * It performs the protocol packet by packet
     * \param packet Buffer of bytes with the packet to be parsed.
     *
     * \return It returns 1 when the processing of the packet leads to a
     * completion of some statement of the protocol. It return 0 otherwise.
     */
    virtual bool parsePacket (unsigned char * packet) = 0;

    /*!
     * It allows access to the content of the data that the last processed
     * packet carrys.
     *
     * \return buffer where the data of the last processed packet is placed.
     */
    const virtual unsigned char * getData () = 0;

};

#endif // PROTOCOL_H
