#ifndef HWDEVICE_H
#define HWDEVICE_H

#include "Property.h"

typedef int HWErrType;

/*!
 * \class HWDevice HWDevice.h
 *
 * \brief Base class for the abstraction of the hardware layer
 *
 */
class HWDevice
{
public:
    /*!
     * It performs the operations for openning the hardware device
     *
     * \return It returns negative number if the hardware can not be opened.
     */
    virtual HWErrType open () = 0;

    /*!
     * It performs the operations for closing the hardware device.
     *
     * \return It returns negative number if the hardware can not be opened.
     */
    virtual HWErrType close () = 0;

    /*!
     * It performs the operations for resetting the hardwre device.
     */
    virtual HWErrType reset () = 0;

    /*!
     * It performs the operations for configuring the hardware device according
     * with the parameters previously configured.
     */
    virtual bool configure () = 0;

    /*!
     * It reads from the hardware device an specific number of bytes.
     *
     * \param buffer Pointer to buffer that receives the data from the device.
     *
     * \param numBytes Number of bytes to be read from the hardware device.
     *
     * \return Number of actual bytes read. If this number does not match with
     * the numBytes parameter, then check the getLastError method.
     */
    virtual int read (unsigned char * buffer, unsigned long numBytes) = 0;

    /*!
     * It writes to the hardware device the number of bytes specified in the
     * parameters.
     */
    virtual int write (unsigned char * buffer, unsigned long numBytes) = 0;

    /*!
     * It returns the last error that might have happened when accessing the
     * hardware device.
     *
     * \return Error type identifier.
     */
    virtual HWErrType getLastError () = 0;
};

#endif // HWDEVICE_H
