#ifndef IDATAPRODUCER_H
#define IDATAPRODUCER_H

#include "PData.h"
#include "IDataConsumer.h"


/*!
 * \class IDataProducer IDataProducer.h
 *
 * \brief Base class of a data producer. It sends data to all the data
 * consumers that are attached to it.
 *
 */
class IDataProducer
{
public:
    /*!
     * Virtual destructor
     */
    virtual ~IDataProducer () {}

    /*!
     * It sends a piece of data to all the data consumers already registered
     *
     * \param data PData object that is sent to the data consumers
     * objects already registered.
     */
    virtual void sendData (const PData& data) = 0;

    /*!
     * It registers a data consumer that is interested on receiving the data
     * that is sent by the data producer
     *
     * \param dataConsumer Data consumer object to be attached to the current
     * data producer.
     */
    virtual void addDataConsumerEndPoint
                            ( IDataConsumer& dataConsumer) = 0;

    /*!
     * It remove a data consumer from the ones already attached to the current
     * data producer.
     *
     * \param dataConsumer Data consumer object to be deregistered.
     */
    virtual void removeDataConsumerEndPoint
                            (const IDataConsumer& dataConsumer) = 0;

};

#endif // IDATAPRODUCER_H
