
#if defined TARGET_HAS_ThirdPartyEnobioAPI
#include "ovasCDriverEnobioAcquisition.h"
#include "ovasCConfigurationEnobioAcquisition.h"

#include <openvibe-toolkit/ovtk_all.h>
#include <Windows.h>
using namespace OpenViBEAcquisitionServer;
using namespace OpenViBE;
using namespace OpenViBE::Kernel;
using namespace std;




//___________________________________________________________________//
//                                                                   //


CDriverEnobioAcquisition::CDriverEnobioAcquisition(IDriverContext& rDriverContext)
:IDriver(rDriverContext)
,m_pCallback(NULL)
,m_ui32SampleCountPerSentBlock(0)
,m_pSample(NULL)
{
	m_oHeader.setSamplingFrequency(250);
	m_oHeader.setChannelCount(4);
}

CDriverEnobioAcquisition::~CDriverEnobioAcquisition(void)
{
}

const char* CDriverEnobioAcquisition::getName(void)
{
	return "Enobio1.6";
}

//___________________________________________________________________//
//                                                                   //

OpenViBE::boolean CDriverEnobioAcquisition::initialize(
	const uint32 ui32SampleCountPerSentBlock,
	IDriverCallback& rCallback)
{

	if(m_rDriverContext.isConnected()) {return false;}
	if(!m_oHeader.isChannelCountSet()||!m_oHeader.isSamplingFrequencySet()) return false;

	// Builds up a buffer to store
	// acquired samples. This buffer
	// will be sent to the acquisition
	// server later...

	m_pSample=new float32[m_oHeader.getChannelCount()*ui32SampleCountPerSentBlock];
	m_errors=new ::StatusData::statusCode[m_EnobioStatusConsumer.buffer_length];
	m_infos=new ::StatusData::statusCode[m_EnobioStatusConsumer.buffer_length];
	if(!m_pSample || !m_errors || !m_infos)
	{
		delete [] m_pSample;
		delete [] m_errors;
		delete [] m_infos;
		m_pSample=NULL;
		m_errors=NULL;
		m_infos=NULL;
		return false;
	}


	// ...
	// initialize hardware and get
	// available header information
	// from it
	// ...
	//register status consumer and data consumer
	m_EnobioDevice.registerConsumer(m_EnobioDevice.ENOBIO_DATA,m_EnobioDataConsumer);
	m_EnobioDevice.registerConsumer(m_EnobioDevice.STATUS,m_EnobioStatusConsumer);
	Property p1("Channel 1 Enabled",1);
	m_EnobioDevice.setProperty(p1);
	Property p2("Channel 2 Enabled",1);
	m_EnobioDevice.setProperty(p2);
	Property p3("Channel 3 Enabled",1);
	m_EnobioDevice.setProperty(p3);
	Property p4("Channel 4 Enabled",1);
	m_EnobioDevice.setProperty(p4);
	m_EnobioDevice.startStreaming();

	::Sleep(1);

	//receive status data
	int err_count=m_EnobioStatusConsumer.consumeErrorData(m_errors);
	int info_count=m_EnobioStatusConsumer.consumeInfoData(m_infos);

	if(err_count>0){
		for(int i=0;i<err_count;i++){		
			string m=::StatusData::getStringFromCode(m_errors[i]);
			m_rDriverContext.getLogManager() << LogLevel_Error << "Problem occured while initializing device. " <<  CString(m.c_str()) <<"\n";
		}
		return false;
	}
	for(int i=0;i<info_count;i++){		
		string m=::StatusData::getStringFromCode(m_infos[i]);
		m_rDriverContext.getLogManager() << LogLevel_Info  <<  CString(m.c_str()) <<"\n";
	}
	// Saves parameters
	m_pCallback=&rCallback;
	m_ui32SampleCountPerSentBlock=ui32SampleCountPerSentBlock;
	return true;
}

OpenViBE::boolean CDriverEnobioAcquisition::start(void)
{
	if(!m_rDriverContext.isConnected()) return false;
	if(m_rDriverContext.isStarted()) return false;

	// ...
	// request hardware to start
	// sending data
	// ...

	m_EnobioDevice.startStreaming();

	//receive status data
	int err_count=m_EnobioStatusConsumer.consumeErrorData(m_errors);
	int info_count=m_EnobioStatusConsumer.consumeInfoData(m_infos);

	if(err_count>0){
		for(int i=0;i<err_count;i++){		
			string m=::StatusData::getStringFromCode(m_errors[i]);
			m_rDriverContext.getLogManager() << LogLevel_Error << "Problem occured while initializing device. " <<  CString(m.c_str()) <<"\n";
		}
		return false;
	}
	for(int i=0;i<info_count;i++){		
		string m=::StatusData::getStringFromCode(m_infos[i]);
		m_rDriverContext.getLogManager() << LogLevel_Info  <<  CString(m.c_str()) <<"\n";
	}
	return true;
}

OpenViBE::boolean CDriverEnobioAcquisition::loop(void)
{
	int count=0;
	if(!m_rDriverContext.isConnected()) return false;
	if(!m_rDriverContext.isStarted()) return true;

	OpenViBE::CStimulationSet l_oStimulationSet;

	// ...
	// receive samples from hardware
	// put them the correct way in the sample array
	// whether the buffer is full, send it to the acquisition server
	//...
	
	while(count<m_ui32SampleCountPerSentBlock){
		//receive enobio data
		count=m_EnobioDataConsumer.consumeData(m_pSample,m_ui32SampleCountPerSentBlock,count);
		//receive status data
		int err_count=m_EnobioStatusConsumer.consumeErrorData(m_errors);
		int info_count=m_EnobioStatusConsumer.consumeInfoData(m_infos);
		
		if(err_count>0){
			for(int i=0;i<err_count;i++){		
				string m=::StatusData::getStringFromCode(m_errors[i]);
				m_rDriverContext.getLogManager() << LogLevel_Error << "Problem occured while initializing device. " <<  CString(m.c_str()) <<"\n";
			}
			return false;
		}
		for(int i=0;i<info_count;i++){		
			string m=::StatusData::getStringFromCode(m_infos[i]);
			m_rDriverContext.getLogManager() << LogLevel_Info  <<  CString(m.c_str()) <<"\n";
		}
	}
	m_pCallback->setSamples(m_pSample);

	// ...
	// receive events from hardware
	// and put them the correct way in a CStimulationSet object
	//...
	m_pCallback->setStimulationSet(l_oStimulationSet);

	return true;
}

OpenViBE::boolean CDriverEnobioAcquisition::stop(void)
{
	if(!m_rDriverContext.isConnected()) return false;
	if(!m_rDriverContext.isStarted()) return false;

	// ...
	// request the hardware to stop
	// sending data
	// ...
	m_EnobioDevice.stopStreaming();
	return true;
}

OpenViBE::boolean CDriverEnobioAcquisition::uninitialize(void)
{
	if(!m_rDriverContext.isConnected()) return false;
	if(m_rDriverContext.isStarted()) return false;

	// ...
	// uninitialize hardware here
	// ...
	m_EnobioDevice.stopStreaming();
	delete [] m_pSample;
	delete [] m_errors;
	delete [] m_infos;
	m_pSample=NULL;
	m_errors=NULL;
	m_infos=NULL;

	m_pCallback=NULL;

	return true;
}

//___________________________________________________________________//
//                                                                   //
OpenViBE::boolean CDriverEnobioAcquisition::isConfigurable(void)
{
	return true; // change to false if your device is not configurable
}

OpenViBE::boolean CDriverEnobioAcquisition::configure(void)
{
	CConfigurationEnobioAcquisition m_oConfiguration(m_rDriverContext,"../share/openvibe-applications/acquisition-server/interface-EnobioAcquisition.ui");
	if(!m_oConfiguration.configure(m_oHeader))
	{
		return false;
	}
	return true;
}
#endif // TARGET_HAS_ThirdPartyEnobioAPI